/* 
 * RafamConsultation
 *
 * Copyright (c) 2010 Office Fdral de l'Informatique et de la Tlcommunication
 * av. Edmond-Vaucher 18
 * Centre Informatique AVS/AI
 * Case Postale 2676
 * 1211 Genve 2
 * 
 * All rights reserved
 * 
 * Creation date Mar Aug 12, 2011
 */
package ch.admin.ofit.ws.client;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.xml.ws.BindingProvider;

import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;

import ch.admin.ofit.alfa.core.ws.domain._2.StandardConsultationRequestType;
import ch.admin.ofit.alfa.core.ws.domain._2.StandardConsultationResponseType;
import ch.admin.ofit.alfa.core.ws.service._2.RAFamConsultation20;
import ch.admin.ofit.alfa.core.ws.service._2.RAFamConsultationService20;
import ch.admin.ofit.commun.ws.domain.ErreurMessageType;
import ch.ech.xmlns.ech_0104._3.DeliveryOfficeType;
import ch.ech.xmlns.ech_0104_69._3.RegisterStatusRecordType;

/**
 * Sample Main class to use RafAmConsultation WebService
 * <p>
 * @author shm, Copyright(c) OFIT - 2011
 * @version $Rev: 1 , $Date: 2010-02-25 18:35:32 +0100 (Thu, 25 Feb 2010) $
 * @since 1.0
 */
public class RafamConsultationSSLSession2 {
    
    /**
     * <code>USAGE</code>System out usage of this class
     */
    private static final String USAGE = "Usage is RafamConsultation url certificatfile certificatepassword vn officeIdentifier [branch]"; //$NON-NLS-1$
    /**
     * <code>url</code>url of webservice
     */
    private static String       url;
    /**
     * <code>certificateFileName</code>Fichier certificat
     */
    private static String       certificateFileName;
    /**
     * <code>certifPassword</code>certificate password to call webservice
     */
    private static String       certifPassword;
    /**
     * <code>vn</code>insure number AVS/AHV
     */
    private static Long         vn;
    /**
     * <code>officeIdentifier</code>delivrery office identifier
     */
    private static String       officeIdentifier;
    /**
     * <code>branch</code>delivrery office branch
     */
    private static Long         branch;
    
    /**
     * Call the webservice with parameters :
     * <ul>
     * <li><b>url</b> : url of webservice. Call famzreg to get it</li>
     * <li><b>certifcatefilename</b> : a full name of certificate file with path
     * for exemple : "c:/Program Files/rafam/ws/certificate.p12" it</li>
     * <li><b>password</b> : password to call webservice. Call famzreg to get it
     * </li>
     * <li><b>vn</b> : insure number AVS/AHV</li>
     * <li><b>officeIdentifier</b> : your</li>
     * <li><b>branch</b> : your delivrery office branch</li>
     * </ul>
     * Example : Run with parameter
     * https://www.infoafam-a.zas.admin.ch/AlfaCore/ws/consultation
     * full-path-file-certificate.p12 certifpassword 7560000410979 2
     * @param args url certifcatefilename certifPassword vn officeIdentifier
     * [branch]
     */
    public static void main(final String[] args) {
        checkParameter(args);
        
        url = args[0];
        certificateFileName = args[1];
        certifPassword = args[2];
        try {
            vn = new Long(args[3]);
        } catch (final NumberFormatException e) {
            System.err.println("The third parameter (vn) is not a number"); //$NON-NLS-1$
            System.err.println(USAGE);
            System.exit(-1);
        }
        officeIdentifier = args[4];
        if (args.length == 6) {
            try {
                branch = new Long(args[5]);
            } catch (final NumberFormatException e) {
                System.err.println("The fifth parameter (branch) is not a number"); //$NON-NLS-1$
                System.err.println(USAGE);
                System.exit(-1);
            }
        } else {
            branch = null;
        }
        findAllocation(url, certificateFileName, certifPassword, vn, officeIdentifier, branch);
    }
    
    /**
     * Check if the number of parameter is right
     * @param args the parameter array
     */
    private static void checkParameter(final String[] args) {
        if (args.length != 5 && args.length != 6) {
            System.err.println("Wrong parameters."); //$NON-NLS-1$
            System.err.println(USAGE);
            System.exit(-1);
        }
    }
    
    /**
     * Find alowance function
     * @param urlRafamWS URL of RAFam webservice
     * @param certFileName certFileName certificat filename with full path
     * @param certPassword certificat password
     * @param nas AHV-Versichertennummer/Numero AVS (13 position)
     * @param cafCode office identifier max 3 positions
     * @param agencyCode delivrery office branch
     */
    private static void findAllocation(final String urlRafamWS, final String certFileName, final String certPassword, final Long nas, final String cafCode,
        final Long agencyCode) {
        
        //1) Create the service
        RAFamConsultation20 rafamConsultationService;
        try {
            rafamConsultationService = getRAFamConsultationService(urlRafamWS, certFileName, certPassword);
        } catch (final Exception e) {
            e.printStackTrace();
            return;
        }
        System.out.println("Create request... "); //$NON-NLS-1$
        // 2) Create the request StandardConsultationInputType
        final StandardConsultationRequestType request = createRequest(nas, cafCode, agencyCode);
        
        System.out.println("Calling RAFamConsultationWebService.findAllocation... "); //$NON-NLS-1$
        //3) Call webservice
        final StandardConsultationResponseType response = rafamConsultationService.findAllocations(request);
        System.out.println("Call successful, result : " + response.getAck().getValue()); //$NON-NLS-1$
        
        //4)  Use the RegisterStatus answer
        for (final RegisterStatusRecordType registerStatus : response.getRegisterStatus()) {
            // Make what you need
            
            // It's an exemple to get information
            final String officeNumber = registerStatus.getDeliveryOffice().getOfficeIdentifier()
                                + (registerStatus.getDeliveryOffice().getBranch() != null ? "." + registerStatus.getDeliveryOffice().getBranch() : ""); //$NON-NLS-1$ //$NON-NLS-2$
            System.out.println("Alowance number " + registerStatus.getRecordNumber() //$NON-NLS-1$
                + " delivrery office " + officeNumber); //$NON-NLS-1$
        }
        //5) Use the Errors
        if (response.getErrors() != null) {
            for (final ErreurMessageType error : response.getErrors().getError()) {
                // Make what you need
                
                // It's an exemple to get error
                System.out.println("Error message : " + error.getMessage()); //$NON-NLS-1$
            }
        }
        System.out.println("End of test."); //$NON-NLS-1$
    }
    
    /**
     * Create the request
     * @param nas AHV-Versichertennummer/Numero AVS (13 position)
     * @param cafCode office identifier max 3 positions
     * @param agencyCode branch office branch max 3 positions
     * @return the request
     */
    private static StandardConsultationRequestType createRequest(final Long nas, final String cafCode, final Long agencyCode) {
        final StandardConsultationRequestType request = new StandardConsultationRequestType();
        request.setNavs(nas.longValue());
        final DeliveryOfficeType caf = new DeliveryOfficeType();
        caf.setOfficeIdentifier(cafCode);
        caf.setBranch(agencyCode);
        request.setDeliveryOffice(caf);
        return request;
    }
    
    /**
     * Create the Service and attach at header request the certificat
     * @param urlRafamWS URL of RAFam webservice
     * @param certFileName certFileName certificat filename with full path
     * @param certPassword certificat password
     * @return RAFamConsultationWebService
     * @throws IOException protocol communication error
     */
    private static RAFamConsultation20 getRAFamConsultationService(final String urlRafamWS, final String certFileName, final String certPassword)
        throws IOException {
        
        final RAFamConsultationService20 RAFamConsultationService = new RAFamConsultationService20();
        final RAFamConsultation20 proxy = RAFamConsultationService.getRAFamConsultation20();
        
        // Set endpoint address (URL) of the webservice
        final Map<String, Object> ctxt = ((BindingProvider) proxy).getRequestContext();
        ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, urlRafamWS);
        
        // Configure SSL On the client         
        configureSSLOnTheClient(proxy, certFileName, certPassword);
        return proxy;
    }
    
    /**
     * Configure SSL On the client
     * @param proxy webservice proxy
     * @param certFileName certFileName certificat filename with full path
     * @param certPassword certificat password
     */
    private static void configureSSLOnTheClient(final RAFamConsultation20 proxy, final String certFileName, final String certPassword) {
        // Get httpConduit
        final Client client = ClientProxy.getClient(proxy);
        final HTTPConduit httpConduit = (HTTPConduit) client.getConduit();
        KeyStore ks = null;
        
        try {
            final TLSClientParameters tlsParams = new TLSClientParameters();
            tlsParams.setDisableCNCheck(true);
            
            // Read the certificate        
            FileInputStream filePkcs12;
            ks = KeyStore.getInstance("PKCS12"); //$NON-NLS-1$
            filePkcs12 = new FileInputStream(certFileName);
            
            // For a better security you can encode your password and decode it here            
            ks.load(filePkcs12, certPassword.toCharArray());
            
            try {
                filePkcs12.close();
            } catch (final IOException e) {
                System.err.println("Error on close " + certFileName + " file"); //$NON-NLS-1$ //$NON-NLS-2$
            }
            
            // Add certificate to the conduit
            final KeyManagerFactory keyFactory = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
            keyFactory.init(ks, certPassword.toCharArray());
            final KeyManager[] km = keyFactory.getKeyManagers();
            tlsParams.setKeyManagers(km);
            
            httpConduit.setTlsClientParameters(tlsParams);
            
        } catch (final FileNotFoundException e) {
            System.err.println("File " + certFileName + " doesn't exist"); //$NON-NLS-1$ //$NON-NLS-2$
        } catch (final IOException ioe) {
            System.err.println("File " + certFileName + " doesn't exist. Cause by " + ioe.getCause()); //$NON-NLS-1$ //$NON-NLS-2$
        } catch (final KeyStoreException kse) {
            System.out.println("Security configuration failed with the following: " + kse.getCause()); //$NON-NLS-1$
        } catch (final NoSuchAlgorithmException nsa) {
            System.out.println("Security configuration failed with the following: " + nsa.getCause()); //$NON-NLS-1$
        } catch (final GeneralSecurityException gse) {
            System.out.println("Security configuration failed with the following: " + gse.getCause()); //$NON-NLS-1$
        }
    }
}
